// reference 10 back-end hosts
//1.upto(10) { n-> host.create("192.168.1.$n",8080) }
hostlist=[]
hostlist+=host.create("localhost",8181)
hostlist+=host.create("localhost",8282)

// load message type definitions from XSD file
prop.inType=datatype.create("test","sample.xsd")
prop.outType=datatype.create("test","sample.xsd")

prop.randomListItem={ lst-> 
  lst=lst.flatten() // workaround for nested lists
  return lst[new Random().nextInt(lst.size())]
}

def serviceList=webservice.build {

  1.upto(20) { i-> //create Service1 .. Service100 
  "Service$i"(delay: 0, failureRate: 0.0) { 

    tags=["worker"]

    //Web service operation "Process"
    Process(input: inType, response: outType) {

      println "Service ${my.webservice.name} call with arguments: $args"
      Thread.sleep(delay)
      if (new Random().nextFloat()<failureRate) {
        println "Service ${my.webservice.name} has thrown error"
        throw new Exception("sorry!")
      }
      println "Service ${my.webservice.name} has finished successfully"
      return outType.newInstance()
    }
  }}

  1.upto(5) { i-> //create 20 delegator services
  "CompositeService$i"() { 

    tags=["delegator","composite"]

    processError={} //initially empty function

    //Web service operation "Delegate"
    Delegate(input: inType, neededResults: hdr(int), 
             response: arrayOf(outType)) {

      println "Service ${my.webservice.name} got arguments: $args"
      def gotResults=0
      def result=[]

      while (gotResults<neededResults) {
        def refs=registry.get{"worker" in it.tags}
        def ref=randomListItem(refs)
        try {
          def resp=ref.Process(input).response
      result+=outType.newInstance().assign(resp)
          gotResults++
          println "Service ${my.webservice.name} got successful result, ole!"
        } catch (e) { 
          println "Service ${my.webservice.name} got error, retry!"
          processError(e) 
        }
      }
      println "Service ${my.webservice.name} has finished successfully"
      return result
    }
  }}
}

serviceList.each { s-> //deploy at random hosts
  println "deploying ${s.name}"
  s.deployAt(randomListItem(hostlist)) 
}



def initClient=client.create()
initClient.run=true //boolean flag 'run'
initClient.code={ //client code as closure
  println "start"
  while (run) {
    Thread.sleep(5000) //every 5 seconds
    def refs=registry.get{"delegator" in it.tags}
    def r=randomListItem(refs) //pick random 
    def arg=inType.newInstance()
    println "Client calling delegator service"
    println r.Delegate(arg, 3) //initiate delegation
    println "Client finished calling delegator service"
  }
}

println "deploying client at all hosts"
initClient.deployAt(hostlist) //run everywhere

// wait 60 secs
println "waiting 60 secs"
Thread.sleep(60*1000)

// then adapt the testbed
def pi=callinterceptor.create()
pi.hooks=[in:"RECEIVE", out :"SEND"] //where to bind
pi.code={ ctx-> println "MESSAGE = ${ctx.soapMsg}" } // just print the msg object

println "adapting running web services"
webservice.call (i:pi) { s-> "delegator" in s.tags} { s-> 
  s.interceptors=[i] //attach to author services
  s.processError= { e->
    // instead of reporting to an external WS (which does not exist)
    // just print out the error message to demonstrate on-the-fly rewriting of functions
    println "OH DEAR: "+e.message
    // def url="http://somehost.com/reportError?WSDL"
    // def reportWs=wsreference.create(url)
    // reportWs.Report(my.webservice.name, e.message)
  }
}

println "changing QoS variables of services"

int cycles=100
while (--cycles>0) {
  Thread.sleep(3000) //every 3 seconds
  println "$cycles cycles until shutdown of testbed execution"
  def workers=webservice.get{"worker" in it.tags}
  def w=randomListItem(workers)
  w.delay=new Random().nextInt(5*1000) //0 - 5sec
  w.failureRate=new Random().nextFloat() //0.0 - 1.0
}


initClient.run=false //shut down all clients

"bye"
