package at.ac.tuwien.vitalab.genesis2.plugin

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement
import at.ac.tuwien.vitalab.genesis2.core.model.types.Host

/**
 * Proxy for Web service calls to remote plugin instances.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class CallProxy {

    private static hostMap=AbstractModelElement.getMap(Host.class)

    private AbstractGenesis2Plugin<?> plugin = null

    private hosts=[]

    public CallProxy(AbstractGenesis2Plugin<?> plugin, Collection<Host> hosts) {
	this.plugin=plugin
	this.hosts=hosts
    }

    public CallProxy(AbstractGenesis2Plugin<?> plugin, Closure filter) {
	this.plugin=plugin
	hosts = hostMap.getAll().grep(filter)
    }

    def getHosts() {
	hosts
    }

    synchronized Map<Host,Object> invokeMethod(final String name, final Object args) {

	def final results=[:]

	def ccc=new ClosureCallCollector()

	if (args.size()==1 && (args[0] instanceof Closure)) {

	    Closure clos=args[0]

	    clos.delegate=ccc
	    clos.resolveStrategy=Closure.DELEGATE_ONLY
	    clos.call(plugin)

	    if (!ccc.calls) {
		return results
	    }
	}

	def final threads=[]

	hosts.each { host ->

	    // run in parrallel threads

	    threads+= Thread.start {

		if (ccc.calls) {
		    def callResults=[]
		    def client
		    try {
			client=getClient(host)
		    } catch (e) {
			results.put(host,e)
			return
		    }

		    ccc.calls.each { callName, callArgs ->
			try {
			    callResults+=[client.invokeMethod(callName,callArgs)]
			} catch (e) {
			    callResults+=e
			}
		    }

		    results.put(host,callResults)
		} else {
		    try {
			def client=getClient(host)
			def res=client.invokeMethod(name,args)

			results.put(host,res)
		    } catch (e) {
			results.put(host,e)
		    }
		}
	    }
	}

	// wait until all have finished
	threads.each { t->
	    t.join()
	}

	return results
    }

    def getClient(host) {
	return plugin.getPluginWsClient(host.host, host.port)
    }

}