package at.ac.tuwien.vitalab.genesis2.base;

import groovy.lang.Closure;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.codehaus.groovy.runtime.MethodClosure;
import org.jdom.JDOMException;

import at.ac.tuwien.vitalab.genesis2.base.ws.Genesis2BaseWebService;
import at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;
import at.ac.tuwien.vitalab.genesis2.core.model.types.DataType;
import at.ac.tuwien.vitalab.genesis2.core.model.types.FakeHost;
import at.ac.tuwien.vitalab.genesis2.core.model.types.Host;
import at.ac.tuwien.vitalab.genesis2.core.model.types.XSD;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureWrapper;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin;

/**
 * Genesis2 Base Plugin.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public class Genesis2Base extends AbstractGenesis2Plugin<Host> {

    /**
     * Plugin name.
     */
    public static final String PLUGINNAME = "Genesis2";

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin#getPluginName
     * ()
     */
    @Override
    public String getPluginName() {
	return PLUGINNAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin#doStart()
     */
    @Override
    public void doStart() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStop()
     */
    @Override
    public void doStop() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin#
     * getPluginWsImplementor()
     */
    @Override
    public Object getPluginWsImplementor() {
	return new Genesis2BaseWebService();
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin#
     * getPluginWsClientClass()
     */
    @Override
    protected Class<?> getPluginWsClientClass() {
	return IGenesis2BaseWebService.class;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin#getAliases()
     */
    @Override
    public List<String> getAliases() {
	Vector<String> aliases = new Vector<String>();
	aliases.add("g2");
	aliases.add("genesis2");
	return aliases;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin#getCommands()
     */
    @Override
    public HashMap<String, Closure> getCommands() {
	HashMap<String, Closure> map = new HashMap<String, Closure>();

	map.put("header", new MethodClosure(this, "createInHeader"));
	map.put("hdr", new MethodClosure(this, "createInHeader"));
	map.put("inHdr", new MethodClosure(this, "createInHeader"));
	map.put("outHdr", new MethodClosure(this, "createOutHeader"));
	map.put("inoutHdr", new MethodClosure(this, "createInOutHeader"));
	map.put("inOutHdr", new MethodClosure(this, "createInOutHeader"));
	map.put("parallel", new MethodClosure(this, "parallel"));

	map.put("arrayOf", new MethodClosure(this, "asArray"));

	map.put("bind", new MethodClosure(this, "bindVariables"));

	return map;
    }

    /*
     * Command.
     */
    public Closure bindVariables(Closure closure, Map<String, Object> vars) {
	Closure clone = (Closure) closure.clone();

	ClosureWrapper.registerBinding(clone, vars);
	return clone;
    }

    /*
     * Command.
     */
    public DataType asArray(DataType dt) throws JDOMException, IOException, ClassNotFoundException {
	return DataType.asArray(dt);
    }

    /**
     * Parallel execution of closures.
     * 
     * @param closures Closures.
     * @return List of results, one per closure.
     */
    public List<Object> parallel(Closure... closures) {

	List<Object> results = new ArrayList<Object>();
	List<Thread> threads = new ArrayList<Thread>();

	// to be able to sort results
	final Map<Thread, Object> resultMap = new HashMap<Thread, Object>();

	for (final Closure c : closures) {

	    Thread t = new Thread() {

		public void run() {
		    try {
			resultMap.put(this, c.call());
		    } catch (Throwable error) {
			resultMap.put(this, error);
		    }
		}

	    };

	    threads.add(t);
	    t.start();
	}

	for (Thread t : threads) {
	    try {
		t.join();
		results.add(resultMap.get(t));
	    } catch (InterruptedException e) {
	    }
	}

	return results;
    }

    /**
     * Transforms a normal data type into a header type.
     * 
     * @param args Arguments for creating the data type.
     * @return Header type.
     */
    public DataType createInHeader(Object... args) {
	DataType dt = AbstractModelElement.getElementMap(DataType.class).create(args);
	return DataType.asHeader(dt, "IN");
    }

    /**
     * Transforms a normal data type into a header type.
     * 
     * @param args Arguments for creating the data type.
     * @return Header type.
     */
    public DataType createOutHeader(Object... args) {
	DataType dt = AbstractModelElement.getElementMap(DataType.class).create(args);
	return DataType.asHeader(dt, "OUT");
    }

    /**
     * Transforms a normal data type into a header type.
     * 
     * @param args Arguments for creating the data type.
     * @return Header type.
     */
    public DataType createInOutHeader(Object... args) {
	DataType dt = AbstractModelElement.getElementMap(DataType.class).create(args);
	return DataType.asHeader(dt, "INOUT");
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#
     * deregisterFromExecutionEnvironment()
     */
    @Override
    protected void deregisterFromExecutionEnvironment() {
	super.deregisterFromExecutionEnvironment();

	ExecutionEnvironment execEnv = ExecutionEnvironment.getGlobalInstance();

	execEnv.setVariable("localhost", null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#
     * registerAtExecutionEnvironment()
     */
    @Override
    protected void registerAtExecutionEnvironment() {
	super.registerAtExecutionEnvironment();

	ExecutionEnvironment execEnv = ExecutionEnvironment.getGlobalInstance();

	execEnv.setVariable("localhost", Host.getLocalhost());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#deployElement(at
     * .ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void deployElement(Host ame) {
	// ignore and do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#undeployElement(
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void undeployElement(Host e) {
	// ignore and do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getArtifact(java
     * .lang.Class, at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * java.lang.String[])
     */
    @Override
    public <S> S getArtifact(Class<S> type, Host modelelement, String... additionalIDs) {
	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getTypesAndBuilders
     * ()
     */
    @Override
    public Map<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> getTypesAndBuilders() {

	HashMap<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> result = new HashMap<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>>();

	result.put(Host.class, null);
	result.put(FakeHost.class, null);
	result.put(DataType.class, null);
	result.put(XSD.class, null);

	return result;

    }

}
