package at.ac.tuwien.vitalab.genesis2.base.ws;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jws.WebService;

import org.apache.log4j.Logger;

import at.ac.tuwien.vitalab.genesis2.base.ExecutionEnvironment;
import at.ac.tuwien.vitalab.genesis2.base.PluginRepositoryMonitor;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.types.Host;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ModelElementWrapper;
import at.ac.tuwien.vitalab.genesis2.errors.G2Exception;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin;

/**
 * Web service of Genesis 2 Base.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@WebService(endpointInterface = "at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService", serviceName = "IGenesis2BaseWebServiceService")
public class Genesis2BaseWebService implements IGenesis2BaseWebService {

    private static final Logger LOGGER = Logger.getLogger(Genesis2BaseWebService.class);

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#ping()
     */
    public String ping() {
	return "pong";
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#getWholeModel
     * ()
     */
    public List<ModelElementWrapper> getWholeModel() {

	List<ModelElementWrapper> result = new ArrayList<ModelElementWrapper>();
	Set<ModelElementWrapper> uniqueKids = new HashSet<ModelElementWrapper>();

	result.add(new ModelElementWrapper(Host.getLocalhost()));

	for (AbstractModelElement kid : Host.getLocalhost().getAllChildren()) {
	    uniqueKids.add(new ModelElementWrapper(kid));
	}

	result.addAll(uniqueKids);

	return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#purge()
     */
    public void purge() {

	// clear all elements
	Host.purgeLocalhost();

	// all variables
	ExecutionEnvironment.clearGlobalInstance();

	// restart plugins
	for (AbstractGenesis2Plugin<?> plugin : AbstractGenesis2Plugin.getRunningPlugins()) {

	    plugin.stop();
	    plugin.start();
	}

	LOGGER.info("Local host has been purged!");
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#listPlugins
     * ()
     */
    public List<String> listPlugins() {

	List<String> names = new ArrayList<String>();

	for (AbstractGenesis2Plugin<?> p : AbstractGenesis2Plugin.getRunningPlugins()) {
	    names.add(p.getPluginName());
	}
	return names;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#
     * listMissingPlugins(java.util.List)
     */
    public List<String> listMissingPlugins(List<String> reqPluginNames) {

	reqPluginNames.removeAll(listPlugins());

	return reqPluginNames;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#installPlugin
     * (java.lang.String, java.lang.String)
     */
    public void installPlugin(String fileName, String jarContentBase64) {

	try {
	    PluginRepositoryMonitor.installPlugin(fileName, jarContentBase64);
	} catch (Exception e) {
	    throw new G2Exception("Cannot install plugin.", e);
	}

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.base.ws.IGenesis2BaseWebService#startPlugin
     * (java.lang.String)
     */
    @Override
    public void startPlugin(String pluginClassName) {

	AbstractGenesis2Plugin.startPlugin(pluginClassName);
    }

}
