package at.ac.tuwien.vitalab.genesis2.base.ws;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebService;

import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ModelElementWrapper;

/**
 * Web Service interface for G2.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@WebService(name = "Genesis2")
public interface IGenesis2BaseWebService {

    /**
     * For testing whether a G2 WS is accessible.
     * 
     * @return "pong".
     */
    @WebMethod
    String ping();

    /**
     * Returns all local model instances.
     * 
     * @return List of wrapped model elements.
     */
    @WebMethod
    List<ModelElementWrapper> getWholeModel();

    /**
     * Cleanup.
     */
    @WebMethod
    void purge();

    /**
     * List all plugin names.
     * 
     * @return List of names.
     */
    @WebMethod
    List<String> listPlugins();

    /**
     * List all plugin names from the passed list, which have not been
     * registered locally.
     * 
     * @param reqPluginNames List of names of required plugins.
     * @return List of missing plugin names.
     */
    @WebMethod
    List<String> listMissingPlugins(List<String> reqPluginNames);

    /**
     * Installs a plugin.
     * 
     * @param fileName Name of the file.
     * @param jarContentBase64 Base64 representation.
     */
    @WebMethod
    void installPlugin(String fileName, String jarContentBase64);

    /**
     * Starts an already installed plugin.
     * 
     * @param pluginClassName Class name of the plugin.
     */
    @WebMethod
    void startPlugin(String pluginClassName);
}
