package at.ac.tuwien.vitalab.genesis2.control;

import groovy.lang.Binding;
import groovy.lang.GroovyShell;

import java.util.UUID;

import lombok.Getter;

import org.codehaus.groovy.control.CompilationFailedException;
import org.codehaus.groovy.control.CompilerConfiguration;

import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureFetchingParser.ClosureFetchingParserFactory;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureWrapper;

/**
 * Groovy Shell which fetches all closure declarations.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public class InterceptingGroovyShell extends GroovyShell {

    protected ClosureFetchingParserFactory factory = null;

    protected int runCounter = 0;

    @Getter
    protected String id = "Genesis" + UUID.randomUUID().toString().replaceAll("-", "");

    private boolean idChanged = false;

    /**
     * Constructor.
     * 
     * @param binding Execution binding.
     * @param config Configuration.
     */
    public InterceptingGroovyShell(Binding binding, CompilerConfiguration config) {
	super(binding, config);
    }

    /**
     * Factory.
     * 
     * @param binding Binding.
     * @return Shell instance.
     */
    public static InterceptingGroovyShell newInstance(Binding binding) {

	ClosureFetchingParserFactory clParserFactory = new ClosureFetchingParserFactory();

	CompilerConfiguration conf = new CompilerConfiguration();
	conf.setPluginFactory(clParserFactory);

	InterceptingGroovyShell shell = new InterceptingGroovyShell(binding, conf);

	shell.factory = clParserFactory;

	return shell;
    }

    /*
     * (non-Javadoc)
     * 
     * @see groovy.lang.GroovyShell#run(java.lang.String, java.lang.String,
     * java.lang.String[])
     */
    @Override
    public Object run(String scriptText, String fileName, String[] args) throws CompilationFailedException {

	if (!idChanged) {
	    id = "ConsoleScript" + (runCounter++);
	}

	factory.reset();

	parse(scriptText);

	ClosureWrapper.registerSources(id, factory.getClosures());

	return super.run(scriptText, fileName, args);
    }

    /*
     * (non-Javadoc)
     * 
     * @see groovy.lang.GroovyShell#evaluate(java.lang.String)
     */
    @Override
    public Object evaluate(String scriptText) throws CompilationFailedException {

	factory.reset();

	Object result = super.evaluate(scriptText);

	ClosureWrapper.registerSources(id, factory.getClosures());

	return result;
    }

    /**
     * Sets the ID.
     * 
     * @param id ID.
     */
    public void setId(String id) {
	this.id = id;
	this.idChanged = true;
    }

    /**
     * Get name of executed script.
     */
    protected synchronized String generateScriptName() {
	return getId() + ".groovy";
    }

}
