package at.ac.tuwien.vitalab.genesis2.core.model.ws;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;

import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureWrapper;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.KeyValueWrapper;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.MapWrapper;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ModelElementWrapper;

/**
 * Web service interface for manipulating model elements.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@WebService(name = "ModelElementService")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, use = SOAPBinding.Use.LITERAL, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public interface IModelElementWebService {

    /**
     * Create a new model element instance.
     * 
     * @param rootElement Root element.
     * @param updatedChildren Its children.
     */
    @WebMethod
    void create(ModelElementWrapper rootElement, List<ModelElementWrapper> updatedChildren);

    /**
     * Remove a model element.
     * 
     * @param id ID.
     */
    @WebMethod
    void remove(String id);

    // +++++++++++++++++++++++++++++++++

    /**
     * Deploy a model element.
     * 
     * @param id ID.
     */
    @WebMethod
    void deployElement(String id);

    /**
     * Deploy a set of model elements.
     * 
     * @param filter Filter closure.
     */
    @WebMethod
    void deployElements(ClosureWrapper filter);

    /**
     * Undeploy a model element.
     * 
     * @param id ID.
     */
    @WebMethod
    void undeployElement(String id);

    /**
     * Undeploy a set of model elements.
     * 
     * @param filter Filter closure.
     */
    @WebMethod
    void undeployElements(ClosureWrapper filter);

    /**
     * Redeploy a model element.
     * 
     * @param id ID.
     */
    @WebMethod
    void redeployElement(String id);

    /**
     * Redeploy a set of model elements.
     * 
     * @param filter Filter closure.
     */
    @WebMethod
    void redeployElements(ClosureWrapper filter);

    // +++++++++++++++++++++++++++++++++

    /**
     * Set a parameter.
     * 
     * @param id Element ID.
     * @param property Property as a key-value tuple.
     */
    @WebMethod
    void setParameter(String id, KeyValueWrapper property);

    /**
     * Upate a set of elements.
     * 
     * @param updatedElements Updated elements.
     */
    @WebMethod
    void update(List<ModelElementWrapper> updatedElements);

    /**
     * Perform arbitraty update.
     * 
     * @param varMap Variable binding for execution.
     * @param filter Filter closure.
     * @param command Command closure.
     */
    @WebMethod
    void change(MapWrapper varMap, ClosureWrapper filter, ClosureWrapper command);

}
