package at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import lombok.Getter;
import lombok.Setter;

/**
 * Wrapper class for collections.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "List")
@XmlRootElement
public class CollectionWrapper extends AbstractWrapper<Collection<?>> {

    /**
     * Type of wrapped collection (class name).
     */
    @XmlAttribute
    @Getter
    @Setter
    protected String type;

    /**
     * List of wrapped objects.
     */
    @XmlElement
    @Getter
    @Setter
    protected List<Object> list;

    /**
     * Constructor.
     */
    public CollectionWrapper() {
	super();
    }

    /**
     * Constructor.
     * 
     * @param col Collection to wrap.
     */
    public CollectionWrapper(Collection<?> col) {
	this(col, null);
    }

    /**
     * Constructor.
     * 
     * @param col Collection to wrap.
     * @param rep Replacer.
     */
    public CollectionWrapper(Collection<?> col, ISerializationObjectReplacer rep) {
	super(col, rep);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AbstractWrapper#
     * wrap(java.lang.Object)
     */
    @Override
    public void wrap(Collection<?> col) {

	if (col == null) {
	    return;
	}

	type = col.getClass().getName();
	list = new ArrayList<Object>();

	for (Object o : col) {
	    list.add(wrapObject(o, replacer));
	}
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AWrapper#unwrap()
     */
    @SuppressWarnings("unchecked")
    @Override
    public Collection<?> unwrap() {

	Collection<Object> result = null;

	if (list != null) {

	    if (type == null) {
		result = new ArrayList<Object>();

	    } else {

		Class<?> clazz;

		try {
		    clazz = Class.forName(type);
		    result = (Collection<Object>) clazz.newInstance();
		} catch (Exception e) {
		    throw new IllegalArgumentException("Wrapped item is not a collection.", e);
		}
	    }

	    for (Object o : list) {
		result.add(unwrapObject(o));
	    }
	}

	return result;
    }

}
