package at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import lombok.Getter;
import lombok.Setter;

/**
 * Wrapper class for key-value tuples.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "KeyValueTuple")
@XmlRootElement
public class KeyValueWrapper extends AbstractWrapper<KeyValue> {

    /**
     * Key.
     */
    @XmlElement
    @Getter
    @Setter
    protected Object key;

    /**
     * Value.
     */
    @XmlElement
    @Getter
    @Setter
    protected Object value;

    /**
     * Constructor.
     */
    public KeyValueWrapper() {
	super();
    }

    /**
     * Constructor.
     * 
     * @param kvObject Key-value tuple to wrap.
     * @param rep Replacer.
     */
    private KeyValueWrapper(KeyValue kvObject, ISerializationObjectReplacer rep) {
	super(kvObject, rep);
    }

    /**
     * Constructor.
     * 
     * @param key Key.
     * @param value Value.
     */
    public KeyValueWrapper(Object key, Object value) {
	this(new KeyValue(key, value), null);
    }

    /**
     * Constructor.
     * 
     * @param key Key.
     * @param value Value.
     * @param rep Replacer.
     */
    public KeyValueWrapper(Object key, Object value, ISerializationObjectReplacer rep) {
	this(new KeyValue(key, value), rep);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AbstractWrapper#
     * wrap(java.lang.Object)
     */
    @Override
    public void wrap(KeyValue kvObject) {

	key = wrapObject(kvObject.key, replacer);
	value = wrapObject(kvObject.value, replacer);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AWrapper#unwrap()
     */
    @Override
    public KeyValue unwrap() {
	return new KeyValue(unwrapObject(key), unwrapObject(value));
    }

    /**
     * Returns the unwrapped key.
     * 
     * @return Key.
     */
    public Object getUnwrappedKey() {
	return unwrapObject(key);
    }

    /**
     * Returns the unwrapped value.
     * 
     * @return Value.
     */
    public Object getUnwrappedValue() {
	return unwrapObject(value);
    }

}
