package at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import lombok.Getter;
import lombok.Setter;

/**
 * Wrapper class for maps.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "Map")
@XmlRootElement
public class MapWrapper extends AbstractWrapper<Map<?, ?>> {

    /**
     * Type of wrapped map (class name).
     */
    @XmlAttribute
    @Getter
    @Setter
    protected String type;

    /**
     * List of key-value tuples, taken from the map.
     */
    @XmlElement
    @Getter
    @Setter
    protected List<KeyValueWrapper> list;

    /**
     * Constructor.
     */
    public MapWrapper() {
	super();
    }

    /**
     * Constructor.
     * 
     * @param map Map to wrap.
     */
    public MapWrapper(Map<?, ?> map) {
	this(map, null);
    }

    /**
     * Constructor.
     * 
     * @param map Map to wrap.
     * @param rep Replacer.
     */
    public MapWrapper(Map<?, ?> map, ISerializationObjectReplacer rep) {
	super(map, rep);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AbstractWrapper#
     * wrap(java.lang.Object)
     */
    @Override
    public void wrap(Map<?, ?> map) {

	if (map == null) {
	    return;
	}

	type = map.getClass().getName();
	list = new ArrayList<KeyValueWrapper>();

	for (Entry<?, ?> entry : map.entrySet()) {
	    list.add(new KeyValueWrapper(entry.getKey(), entry.getValue(), replacer)); // NOPMD
	}

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AbstractWrapper#
     * unwrap()
     */
    @SuppressWarnings("unchecked")
    @Override
    public Map<?, ?> unwrap() {

	Map<Object, Object> result = null;

	if (list != null) {

	    if (type == null) {
		result = new LinkedHashMap<Object, Object>();

	    } else {
		Class<?> clazz;

		try {
		    clazz = Class.forName(type);
		    result = (Map<Object, Object>) clazz.newInstance();
		} catch (Exception e) {
		    throw new IllegalArgumentException("Wrapped item is not a map.", e);
		}
	    }

	    for (KeyValueWrapper kvWrapper : list) {
		result.put(unwrapObject(kvWrapper.key), unwrapObject(kvWrapper.value));
	    }
	}

	return result;
    }

}
