package at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers;

import java.util.HashMap;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import lombok.Getter;
import lombok.Setter;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.ElementMap;

/**
 * Wrapper class for model element references.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "ElementReference")
@XmlRootElement
public class ModelElementReferenceWrapper extends AbstractWrapper<AbstractModelElement> {

    /**
     * Element type (class name).
     */
    @XmlAttribute
    @Getter
    @Setter
    private String type;

    /**
     * Element id.
     */
    @XmlAttribute
    @Getter
    @Setter
    private String id; // NOPMD

    /**
     * Constructor.
     */
    public ModelElementReferenceWrapper() {
	super();
    }

    /**
     * Constructor.
     * 
     * @param element Model element.
     * @param rep Replacer.
     */
    public ModelElementReferenceWrapper(AbstractModelElement element, ISerializationObjectReplacer rep) {
	super(element, rep);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AbstractWrapper#
     * wrap(java.lang.Object)
     */
    @Override
    public void wrap(AbstractModelElement element) {

	if (element != null) {
	    type = element.getClass().getSimpleName();
	    id = element.getId();
	}
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AWrapper#unwrap()
     */
    @Override
    public AbstractModelElement unwrap() {

	if (type == null && id == null) {
	    return null; // NOPMD
	}

	ElementMap<? extends AbstractModelElement> map = null;

	if (type != null) {
	    map = AbstractModelElement.getElementMap(type);
	}

	if (map == null) {
	    throw new IllegalArgumentException("Unknown model type '" + type + "'.");
	}

	if (id == null) {
	    throw new IllegalArgumentException("Missing ID String.");
	}

	AbstractModelElement element = map.get(id);

	if (element == null) {

	    // if null, create a new instance
	    HashMap<String, Object> props = new HashMap<String, Object>();

	    props.put("id", id);
	    props.put("type", type);

	    element = map.create(props);
	}

	return element;

    }

    /**
     * @author Lukasz Juszczyk
     * 
     */
    public static class ModelElementReferenceReplacer implements ISerializationObjectReplacer {

	/*
	 * (non-Javadoc)
	 * 
	 * @see at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.
	 * ISerializationObjectReplacer#replace(java.lang.Object,
	 * at.ac.tuwien.vitalab
	 * .genesis2.core.model.ws.wrappers.ISerializationObjectReplacer)
	 */
	@Override
	public Object replace(Object value, ISerializationObjectReplacer rep) {

	    Object result = value;

	    if ((value != null) && (value instanceof AbstractModelElement)) {
		result = new ModelElementReferenceWrapper((AbstractModelElement) value, rep);
	    }
	    return result;
	}
    }

}
