package at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers;

import java.util.Map;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import lombok.Getter;
import lombok.Setter;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.ElementMap;

/**
 * Wrapper class for model elements.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "ModelElement")
@XmlRootElement
public class ModelElementWrapper extends AbstractWrapper<AbstractModelElement> {

    /**
     * All wrapped properties of the model element.
     */
    @XmlElement
    @Getter
    @Setter
    private MapWrapper properties;

    /**
     * Constructor.
     */
    public ModelElementWrapper() {
	super();
    }

    /**
     * Constructor.
     * 
     * @param element Model element.
     */
    public ModelElementWrapper(AbstractModelElement element) {
	super(element, null);
    }

    /**
     * Constructor.
     * 
     * @param element Model element.
     * @param rep Replacer.
     */
    public ModelElementWrapper(AbstractModelElement element, ISerializationObjectReplacer rep) {
	super(element, rep);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AbstractWrapper#
     * wrap(java.lang.Object)
     */
    @Override
    public void wrap(AbstractModelElement element) {

	if (element != null) {
	    properties = new MapWrapper(element.getProperties(), replacer);
	}
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.AWrapper#unwrap()
     */
    @Override
    public AbstractModelElement unwrap() {

	Map<?, ?> propMap = (Map<?, ?>) properties.unwrap();

	String type = (String) propMap.get("type");
	String id = (String) propMap.get("id"); // NOPMD

	ElementMap<? extends AbstractModelElement> elementMap = AbstractModelElement.getElementMap(type);

	if (elementMap == null) {
	    throw new IllegalArgumentException("Unknown type '" + type + "'");
	}

	AbstractModelElement element = elementMap.get(id);

	if (element == null) {
	    element = elementMap.create(propMap);
	} else {
	    updateProperties(element, propMap);
	}

	return element;

    }

    /**
     * Update properties of a model element.
     * 
     * @param element Model element.
     * @param props Properties.
     */
    private synchronized final void updateProperties(AbstractModelElement element, Map<?, ?> props) { // NOPMD

	// first remove all deleted props
	for (Object propKey : element.getProperties().keySet()) {
	    if (!props.containsKey(propKey)) {
		element.setProperty(propKey.toString(), null);
	    }
	}

	// and update changed ones
	for (Object propKey : props.keySet()) {

	    String propName = propKey.toString();

	    if ("id".equals(propName)) {
		continue;
	    }

	    Object propValue = props.get(propName);

	    element.setProperty(propName, propValue);
	}
    }

}
