package at.ac.tuwien.vitalab.genesis2.plugin.callinterceptor;

import groovy.lang.Closure;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.cxf.endpoint.Endpoint;

import at.ac.tuwien.vitalab.genesis2.base.ExecutionEnvironment;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.ElementMap;
import at.ac.tuwien.vitalab.genesis2.core.model.IModelObserver;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractStandaloneGenesis2Plugin;
import at.ac.tuwien.vitalab.genesis2.plugin.wsgen.WebService;
import at.ac.tuwien.vitalab.genesis2.plugin.wsgen.WebServiceGeneratorPlugin;

/**
 * Plugin for intercepting SOAP calls.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@SuppressWarnings("PMD.UncommentedEmptyMethod")
public class CallInterceptorPlugin extends AbstractStandaloneGenesis2Plugin<CallInterceptor> {

    /**
     * Plugin name.
     */
    public static final String PLUGINNAME = "CallInterceptor";

    private IModelObserver<WebService> serviceObserver = null;
    private ElementMap<WebService> serviceMap = null;

    private Map<WebService, Collection<CallInterceptor>> deployedInterceptors = new HashMap<WebService, Collection<CallInterceptor>>();

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getPluginName()
     */
    @Override
    public String getPluginName() {
	return PLUGINNAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getTypesAndBuilders
     * ()
     */
    @Override
    public Map<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> getTypesAndBuilders() {

	HashMap<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> result = new HashMap<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>>();

	result.put(CallInterceptor.class, CallInterceptorBuilder.class);

	return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#deployElement(at
     * .ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void deployElement(CallInterceptor e) {
	// nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getArtifact(java
     * .lang.Class, at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * java.lang.String[])
     */
    @Override
    public <S> S getArtifact(Class<S> type, CallInterceptor modelelement, String... additionalIDs) {
	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStart()
     */
    @Override
    protected void doStart() {

	serviceMap = AbstractModelElement.getElementMap(WebService.class);
	serviceObserver = new ServiceObserver();
	serviceMap.observe(serviceObserver);
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStop()
     */
    @Override
    protected void doStop() {

	serviceMap.stopObserving(serviceObserver);
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getAliases()
     */
    @Override
    public List<String> getAliases() {
	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getCommands()
     */
    @Override
    public Map<String, Closure> getCommands() {
	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#undeployElement(
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void undeployElement(CallInterceptor e) {
	// nothing
    }

    /**
     * Watches for deployed web services and attaches itself to them.
     * 
     * @author Lukasz Juszczyk
     * 
     */
    protected class ServiceObserver implements IModelObserver<WebService> {

	/*
	 * (non-Javadoc)
	 * 
	 * @see at.ac.tuwien.vitalab.genesis2.core.model.IModelObserver#
	 * changedModelElement
	 * (at.ac.tuwien.vitalab.genesis2.core.model.AModelElement, int,
	 * java.lang.Object[])
	 */
	@Override
	public void onChangedModelElement(WebService service, int how, Object... info) {

	    try {

		switch (how) {

		case IModelObserver.UPDATED:

		    if (!"interceptors".equals(info[0])) {
			break;
		    }

		case IModelObserver.DEPLOYED:

		    deployServiceInterceptors(service);

		    break;

		case IModelObserver.UNDEPLOYED:

		    undeployServiceInterceptors(service);

		    break;
		}

	    } catch (Exception e) {
		logger.warn(e);
	    }

	}
    }

    /**
     * Attaches itself to a Web service.
     * 
     * @param service Web service.
     */
    protected void deployServiceInterceptors(WebService service) {

	WebServiceGeneratorPlugin wsgenPlugin = AbstractGenesis2Plugin.getPlugin(WebServiceGeneratorPlugin.class);

	if (service == null || wsgenPlugin == null) {
	    return;
	}

	Endpoint endpoint = wsgenPlugin.getArtifact(Endpoint.class, service);

	if (endpoint == null) {
	    return;
	}

	Collection<CallInterceptor> newInterceptors = (Collection<CallInterceptor>) service.getProperty("interceptors");

	// if already deployed, remove it
	if (deployedInterceptors.get(service) != null) {
	    for (CallInterceptor oldInter : deployedInterceptors.get(service)) {

		oldInter.detachFromEndpoint(endpoint);
	    }
	}

	// attach all current interceptors
	if (newInterceptors != null) {
	    for (CallInterceptor newInter : newInterceptors) {

		ExecutionEnvironment env = new ExecutionEnvironment();
		env.setProperty("global", wsgenPlugin.getServiceVariableBinding(service));

		newInter.attachToEndpoint(endpoint, service, env);
	    }
	}

	if (newInterceptors != null) {
	    deployedInterceptors.put(service, new Vector<CallInterceptor>(newInterceptors));
	}
    }

    /**
     * Detaches itself from a Web service.
     * 
     * @param service Web service.
     */
    protected void undeployServiceInterceptors(WebService service) {

	WebServiceGeneratorPlugin wsgenPlugin = AbstractGenesis2Plugin.getPlugin(WebServiceGeneratorPlugin.class);

	if (service == null || wsgenPlugin == null) {
	    return;
	}

	Endpoint endpoint = wsgenPlugin.getArtifact(Endpoint.class, service);

	if (endpoint == null) {
	    return;
	}

	for (CallInterceptor oldInter : deployedInterceptors.get(service)) {
	    oldInter.detachFromEndpoint(endpoint);
	}

	deployedInterceptors.remove(service);
    }

}
