package at.ac.tuwien.vitalab.genesis2.plugin.clientgen;

import groovy.lang.Closure;

import java.util.Collection;

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.types.Host;

/**
 * Client model element.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public class Client extends AbstractModelElement {

    /**
     * Constructor.
     */
    public Client() {
	super();
    }

    /**
     * Constructor.
     * 
     * @param code Client code.
     */
    public Client(Closure code) {
	this();
	setCode(code);
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.core.model.ICleanable#cleanup()
     */
    @Override
    public void cleanup() {
    }

    /**
     * Getter for the code closure.
     * 
     * @return Closure.
     */
    public Closure getCode() {
	return (Closure) getProperty("code");
    }

    /**
     * Setter for the code closure.
     * 
     * @param code Closure.
     */
    public final void setCode(Closure code) {
	setProperty("code", code);
    }

    /**
     * Command alias run -> deploy.
     */
    public void run() {
	this.deploy();
    }

    /**
     * Command alias run -> deploy.
     * 
     * @param hosts Host list.
     */
    public void runAt(Host... hosts) {
	this.deployAt(hosts);
    }

    /**
     * Command alias run -> deploy.
     * 
     * @param hosts Host list.
     */
    public void runAt(Collection<Host> hosts) {
	this.deployAt(hosts);
    }

    /**
     * Command alias stop -> undeployFrom.
     */
    public void stop() {
	this.undeployFrom();
    }

    /**
     * Command alias stop -> undeployFrom.
     * 
     * @param hosts Host list.
     */
    public void stopAt(Host... hosts) {
	this.undeployFrom(hosts);
    }

    /**
     * Command alias stop -> undeployFrom.
     * 
     * @param hosts Host list.
     */
    public void stopAt(Collection<Host> hosts) {
	this.undeployFrom(hosts);
    }

}
