package at.ac.tuwien.vitalab.genesis2.plugin.msgpert;

import groovy.lang.Closure;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.codehaus.groovy.runtime.InvokerHelper;
import org.codehaus.groovy.runtime.MethodClosure;
import org.w3c.dom.Node;

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractStandaloneGenesis2Plugin;

/**
 * Plugin for message perturbation/corruption.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@SuppressWarnings("PMD.UncommentedEmptyMethod")
public class MessagePerturbationPlugin extends AbstractStandaloneGenesis2Plugin<MsgPertInterceptor> {

    /**
     * Plugin name.
     */
    public static final String PLUGIN_NAME = "MessagePerturbationPlugin";

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#deployElement(at
     * .ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void deployElement(MsgPertInterceptor e) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStart()
     */
    @Override
    protected void doStart() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStop()
     */
    @Override
    protected void doStop() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getAliases()
     */
    @Override
    public List<String> getAliases() {
	List<String> aliases = new ArrayList<String>();
	aliases.add("msgpert");
	return aliases;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getCommands()
     */
    @Override
    public Map<String, Closure> getCommands() {
	Map<String, Closure> commands = new HashMap<String, Closure>();
	commands.put("visit", new MethodClosure(this, "visit"));
	return commands;
    }

    /**
     * Apply the closure recursively to all elements. (Visitor pattern).
     * 
     * @param closure Closure.
     * @param el Root element.
     */
    public static void visitXML(Closure closure, Node el) {

	Closure cl = (Closure) closure.clone();
	cl.setDelegate(el);

	try {
	    cl.call(el);
	} catch (Exception e) {
	    // ignore exceptions
	}

	for (int i = 0; i < el.getChildNodes().getLength(); i++) {
	    Node ch = el.getChildNodes().item(i);
	    visitXML(closure, ch);
	}
    }

    /**
     * Apply the closure recursively to all fields of the object. (Visitor
     * pattern).
     * 
     * @param closure Closure.
     * @param o Root object.
     */
    public static void visitArgs(Closure closure, Object o) {
	visitItem(closure, new Item("root", null, o, null), new HashSet<Object>());
    }

    /**
     * Apply the closure recursively to the item and its children. (Visitor
     * pattern).
     * 
     * @param closure Closure.
     * @param item Item
     * @param beenThere Set of already visited items.
     */
    private static void visitItem(Closure closure, Item item, Set<Object> beenThere) {

	if (item == null || beenThere.contains(item)) {
	    return;
	}

	Closure cl = (Closure) closure.clone();
	cl.setDelegate(item);

	try {
	    cl.call(item);
	} catch (Exception e) {
	    // ignore exceptions
	}

	// checking for lists/collections is sufficient.
	// the unmarshalled arguments will not have maps
	if (item.value instanceof Collection) {
	    Collection<?> col = (Collection<?>) item.value;

	    for (Object co : new ArrayList<Object>(col)) {

		Item colItem = new Item(item.name + "_item", item.value, co, null);

		visitItem(closure, colItem, beenThere);
	    }
	}

	for (Method m : item.type.getMethods()) {

	    try {
		if (m.getName().startsWith("get") && m.getParameterTypes().length == 0) {

		    if (m.getName().equals("getClass")) {
			continue;
		    }

		    Class<?> type = m.getReturnType();
		    Object val = m.invoke(item.value);

		    String name = m.getName().substring(3);
		    if (name.length() > 1) {
			name = Character.toLowerCase(name.charAt(0)) + name.substring(1);
		    } else {
			name = "" + Character.toLowerCase(name.charAt(0));
		    }

		    Set<Object> newBeenThere = new HashSet<Object>(beenThere);
		    newBeenThere.add(item.value);

		    Item viaGetterItem = new Item(name, item.value, val, type);
		    visitItem(closure, viaGetterItem, newBeenThere);

		    if (List.class.isAssignableFrom(type) && val != null) {
			for (Object o : (List<?>) val) {
			    visitItem(closure, new Item(name + "_item", val, o, null), newBeenThere);
			}
		    }
		}

	    } catch (Exception e) {
		// skip it...
	    }
	}
    }

    /**
     * Item for encapsulating values of an unmarshalled message object.
     * 
     * @author Lukasz Juszczyk
     * 
     */
    protected static class Item {

	public String name;
	public String typename;
	public String simpleTypename;
	public Class<?> type;
	public Object value;
	public Object parent;

	/**
	 * Constructor.
	 * 
	 * @param name Name.
	 * @param parent Parent object.
	 * @param obj Processed object.
	 * @param clazz Class.
	 */
	public Item(String name, Object parent, Object obj, Class<?> clazz) {
	    this.parent = parent;
	    this.name = name;
	    this.value = obj;

	    if (clazz == null) {
		this.type = obj.getClass();
	    } else {
		this.type = clazz;
	    }

	    this.typename = this.type.getName();
	    this.simpleTypename = this.type.getSimpleName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
	    return name + "(" + type + ")=" + value;
	}

	/**
	 * Generic setter.
	 * 
	 * @param val Value.
	 */
	public void set(Object val) {

	    if (parent == null) {
		return;
	    }

	    if (parent instanceof List) {
		@SuppressWarnings("unchecked")
		List<Object> list = (List<Object>) parent;

		int indexOf = list.indexOf(this.value);

		if (indexOf >= 0) { // replace
		    list.remove(indexOf);
		    if (val != null) {
			list.add(indexOf, val);
		    }
		} else { // or add
		    list.add(val);
		}

	    } else {
		try {
		    InvokerHelper.setProperty(parent, name, val);
		} catch (Exception e) {
		}
	    }

	    value = val;
	}
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getPluginName()
     */
    @Override
    public String getPluginName() {
	return PLUGIN_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getTypesAndBuilders
     * ()
     */
    @Override
    public Map<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> getTypesAndBuilders() {
	Map<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> result = new HashMap<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>>();

	result.put(MsgPertInterceptor.class, null);

	return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#undeployElement(
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void undeployElement(MsgPertInterceptor e) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getArtifact(java
     * .lang.Class, at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * java.lang.String[])
     */
    @Override
    public <S> S getArtifact(Class<S> type, MsgPertInterceptor modelelement, String... additionalIDs) {
	return null;
    }

}
