package at.ac.tuwien.vitalab.genesis2.plugin.qosemu;

import groovy.lang.Closure;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;

/**
 * Model tyoe for specifying QOS properties.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@SuppressWarnings("javadoc")
public class QOSProps extends AbstractModelElement {

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.core.model.ICleanable#cleanup()
     */
    @Override
    public void cleanup() {

    }

    /**
     * Gets the value of a property, executing closures if these are present.
     * 
     * @param name Property name.
     * @param defaultValue Default value.
     * @return Value.
     */
    private Number getValue(String name, Number defaultValue) {

	Object val = getProperty(name);

	if (val == null) {
	    return defaultValue;
	}

	if (val instanceof Closure) {
	    Closure c = (Closure) val;

	    val = c.call();
	}

	if (val instanceof Number) {
	    return (Number) val;
	}

	return defaultValue;
    }

    public double getAvailability() {
	return getValue("availability", 1.0).doubleValue();
    }

    public void setAvailability(Object availability) {
	setProperty("availability", availability);
    }

    public double getErrorRate() {
	return getValue("errorRate", 1.0).doubleValue();
    }

    public void setErrorRate(Object errorRate) {
	setProperty("errorRate", errorRate);
    }

    public int getResponseTime() {
	return getValue("responseTime", 0).intValue();
    }

    public void setProcessingTime(Object responseTime) {
	setProperty("responseTime", responseTime);
    }

    public int getThroughput() {
	return getValue("throughput", -1).intValue();
    }

    public void setThroughput(Object throughput) {
	setProperty("throughput", throughput);
    }

    public double getScalability() {
	return getValue("scaleability", 1).doubleValue();
    }

    public void setScalability(Object scalability) {
	setProperty("scaleability", scalability);
    }
}
