package at.ac.tuwien.vitalab.genesis2.plugin.registry;

import groovy.lang.Closure;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureWrapper;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ModelElementWrapper;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractGenesis2Plugin;
import at.ac.tuwien.vitalab.genesis2.plugin.hostlist.HostListPlugin;
import at.ac.tuwien.vitalab.genesis2.plugin.registry.ws.IRegistryWebService;
import at.ac.tuwien.vitalab.genesis2.plugin.registry.ws.RegistryWebService;
import at.ac.tuwien.vitalab.genesis2.plugin.wsreference.WsReference;

/**
 * Simple Broadcasting Registry.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@SuppressWarnings("PMD.UncommentedEmptyMethod")
public class WebServiceRegistryPlugin extends AbstractGenesis2Plugin<AbstractModelElement> {

    /**
     * Plugin name.
     */
    public static final String PLUGINNAME = "SimpleWebServiceRegistry";

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStart()
     */
    @Override
    protected void doStart() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStop()
     */
    @Override
    protected void doStop() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getAliases()
     */
    @Override
    public List<String> getAliases() {
	List<String> result = new ArrayList<String>();
	result.add("registry");
	return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getCommands()
     */
    @Override
    public Map<String, Closure> getCommands() {
	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getPluginName()
     */
    @Override
    public String getPluginName() {
	return PLUGINNAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getPluginWsClientClass
     * ()
     */
    @Override
    protected Class<?> getPluginWsClientClass() {
	return IRegistryWebService.class;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getPluginWsImplementor
     * ()
     */
    @Override
    public Object getPluginWsImplementor() {
	return new RegistryWebService();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getTypesAndBuilders
     * ()
     */
    @Override
    public Map<Class<? extends at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement>>> getTypesAndBuilders() {

	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#deployElement(at
     * .ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void deployElement(AbstractModelElement e) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#undeployElement(
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void undeployElement(AbstractModelElement e) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getArtifact(java
     * .lang.Class, at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * java.lang.String[])
     */
    @Override
    public <S> S getArtifact(Class<S> type, AbstractModelElement modelelement, String... additionalIDs) {
	return null;
    }

    /**
     * Lookup.
     * 
     * @param filter Filter closure.
     * @param props Names of additional properties which will be retrieved.
     * 
     * @return List of references.
     */
    public ArrayList<WsReference> get(Closure filter, final String... props) {

	final ClosureWrapper wrapper = new ClosureWrapper(filter);
	final List<WsReference> result = Collections.synchronizedList(new ArrayList<WsReference>());

	HostListPlugin hostlist = AbstractGenesis2Plugin.getPlugin(HostListPlugin.class);

	List<Thread> threads = new ArrayList<Thread>();

	synchronized (hostlist) {
	    for (final String hostAndPort : hostlist.getList()) {

		Thread t = new Thread() {
		    public void run() {
			String host = HostListPlugin.getHost(hostAndPort);
			int port = HostListPlugin.getPort(hostAndPort);

			IRegistryWebService ws = (IRegistryWebService) getPluginWsClient(host, port);

			try {

			    List<String> reqNames = new ArrayList<String>();

			    reqNames.add("name"); // mandatory

			    for (String p : props) {
				reqNames.add(p);
			    }

			    List<ModelElementWrapper> wrappedRefs = ws.get(wrapper, reqNames);

			    if (wrappedRefs != null) {
				for (ModelElementWrapper wrap : wrappedRefs) {

				    try {
					WsReference ref = (WsReference) wrap.unwrap();
					result.add(ref);
				    } catch (ClassCastException cce) {
					continue;
				    }

				}
			    }
			} catch (Exception e) {
			    logger.warn("Cannot contact registry at " + hostAndPort + ".", e);
			}
		    }
		};

		threads.add(t);
		t.start();
	    }
	}

	for (Thread t : threads) {
	    try {
		t.join();
	    } catch (InterruptedException e) {
	    }
	}

	return new ArrayList<WsReference>(result);
    }
}
