package at.ac.tuwien.vitalab.genesis2.plugin.registry.ws;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.ElementMap;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureWrapper;
import at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ModelElementWrapper;
import at.ac.tuwien.vitalab.genesis2.plugin.wsgen.WebService;
import at.ac.tuwien.vitalab.genesis2.plugin.wsreference.WsReference;

/**
 * Registry Web service providing lookups by filter closures.
 * 
 * @author Lukasz Juszczyk
 * 
 */
@javax.jws.WebService(endpointInterface = "at.ac.tuwien.vitalab.genesis2.plugin.registry.ws.IRegistryWebService", serviceName = "IRegistryWebService")
public class RegistryWebService implements IRegistryWebService {

    private final ElementMap<WebService> serviceMap = AbstractModelElement.getElementMap(WebService.class);

    private static final Logger LOGGER = Logger.getLogger(RegistryWebService.class);

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.registry.ws.IRegistryWebService#
     * get(at.ac.tuwien.vitalab.genesis2.core.model.ws.wrappers.ClosureWrapper,
     * java.util.List)
     */
    @Override
    public List<ModelElementWrapper> get(ClosureWrapper filter, List<String> requiredProps) {

	try {

	    List<ModelElementWrapper> list = new ArrayList<ModelElementWrapper>();

	    Collection<WebService> services = serviceMap.get(filter.unwrap());

	    for (WebService s : services) {

		if (!s.isLocallyDeployed()) {
		    continue;
		}

		String endpointURL = s.getLocalEndpointURL();
		String wsdlURL = endpointURL + "?WSDL";

		WsReference ref = new WsReference(wsdlURL, endpointURL);

		if (requiredProps != null) {
		    for (String prop : requiredProps) {
			ref.setProperty(prop, s.getProperty(prop));
		    }
		}

		list.add(new ModelElementWrapper(ref));
	    }

	    return list;

	} catch (Exception e) {
	    LOGGER.debug("Registry cannot process request. ", e);

	    return null;
	}
    }
}
