package at.ac.tuwien.vitalab.genesis2.plugin.wsgen;

import groovy.lang.Closure;

import java.util.Map;

import org.apache.log4j.Logger;

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;

/**
 * Builder for the WebService type.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public class WebServiceBuilder extends AbstractModelBuilder<WebService> {

    private static final Logger LOGGER = Logger.getLogger(WebServiceBuilder.class);

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.builder.AModelBuilder#attachChild
     * (at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    protected void attachOrphanedChild(WebService parent, AbstractModelElement child) {

	if (parent == null || child == null) {
	    return;
	}

	if (child instanceof WsOperation) {
	    parent.addOperation((WsOperation) child);
	    return;
	}

	throw new IllegalArgumentException("Type '" + child.getClass().getName() + "' cannot be attached to type '"
		+ parent.getClass().getName() + "'.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.builder.AModelBuilder#nextChildType
     * (java.lang.Object, java.lang.Object, java.util.Map)
     */
    @Override
    protected Class<? extends AbstractModelElement> nextChildType(Object name, Object value, Map<?, ?> attributes) {
	return WsOperation.class;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.builder.AModelBuilder#invokeMethod
     * (java.lang.String, java.lang.Object)
     */
    @Override
    public Object invokeMethod(String methodName, Object args) {

	if ("callbackOf".equals(methodName)) {
	    try {
		Object[] argArray = (Object[]) args;

		WsOperation op = (WsOperation) argArray[0];
		Closure cl = (Closure) argArray[1];

		WsOperation callback = WsOperation.getCallbackOf(op, cl);

		kids.add(callback);

		return callback;

	    } catch (ClassCastException cce) {
		LOGGER.warn(cce);
		// ignore
	    } catch (ArrayIndexOutOfBoundsException ae) {
		LOGGER.warn(ae);
		// ignore
	    }

	} else if ("faultCallbackOf".equals(methodName)) {
	    try {
		Object[] argArray = (Object[]) args;

		WsOperation op = (WsOperation) argArray[0];
		Closure cl = (Closure) argArray[1];

		WsOperation callback = WsOperation.faultCallbackOf(op, cl);

		kids.add(callback);

		return callback;

	    } catch (ClassCastException cce) {
		LOGGER.warn(cce);
		// ignore
	    } catch (ArrayIndexOutOfBoundsException ae) {
		LOGGER.warn(ae);
		// ignore
	    }

	}

	return super.invokeMethod(methodName, args);
    }
}
