package at.ac.tuwien.vitalab.genesis2.plugin.wsgen;

import groovy.lang.Binding;
import groovy.lang.Closure;
import groovy.lang.GroovyShell;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.ws.WebServiceContext;

import org.apache.cxf.endpoint.Endpoint;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.codehaus.groovy.runtime.InvokerHelper;

import at.ac.tuwien.vitalab.genesis2.base.ExecutionEnvironment;
import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;
import at.ac.tuwien.vitalab.genesis2.core.model.types.DataType;
import at.ac.tuwien.vitalab.genesis2.core.model.types.Host;
import at.ac.tuwien.vitalab.genesis2.errors.G2Exception;
import at.ac.tuwien.vitalab.genesis2.plugin.AbstractStandaloneGenesis2Plugin;
import at.ac.tuwien.vitalab.genesis2.utils.ClosureUtil;

/**
 * SOAP Web Service Generator Plugin.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public class WebServiceGeneratorPlugin extends AbstractStandaloneGenesis2Plugin<WebService> {

    /**
     * Plugin name.
     */
    public static final String PLUGINNAME = "WebServiceGenerator";

    private final Map<WebService, Object> deployedServices = new HashMap<WebService, Object>();
    private final Map<WebService, Server> deployedEndpoints = new HashMap<WebService, Server>();
    private final Map<WebService, Binding> serviceVarBindings = new HashMap<WebService, Binding>();

    private static final ClosureUtil CLOSURE_UTIL = ClosureUtil.getInstance();

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getPluginName()
     */
    @Override
    public String getPluginName() {
	return PLUGINNAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#deployElement(at
     * .ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void deployElement(WebService wsm) {

	undeployElement(wsm);

	Object ws;
	try {
	    ws = createWsImplementor(wsm);
	} catch (RuntimeException re) {
	    throw re;
	} catch (Exception e) {
	    throw new G2Exception(e);
	}

	JaxWsServerFactoryBean svrFactory = new JaxWsServerFactoryBean();
	svrFactory.setAddress(wsm.getLocalEndpointURL());
	svrFactory.setServiceBean(ws);
	Server s = svrFactory.create();

	deployedEndpoints.put(wsm, s);
	deployedServices.put(wsm, ws);
    }

    /**
     * Generates and compiles the class which implements the modelled Web
     * service.
     * 
     * @param wsm Web service model.
     * @return Implementor object.
     * @throws Exception On errors.
     */
    private Object createWsImplementor(AbstractModelElement wsm) throws Exception {

	/*
	 * I KNOW THIS METHOD IS HIDEOS AND SHOULD BE REPLACED BY USING A
	 * TEMPLATING ENGINE! THIS METHOD HAS CONSTANTLY GROWN SINCE THE VERY
	 * BEGINNING AND SO FAR I HAVE BEEN TOO LAZY TO REWRITE IT.
	 * 
	 * SORRY!
	 */

	HashSet<String> tabuList = new HashSet<String>();

	final WebService service = (WebService) wsm;

	StringWriter classStringWriter = new StringWriter();
	PrintWriter classWriter = new PrintWriter(classStringWriter);

	StringWriter ifaceStringWriter = new StringWriter();
	PrintWriter ifaceWriter = new PrintWriter(ifaceStringWriter);

	ifaceWriter.println("import javax.jws.*;");
	ifaceWriter.println("import javax.xml.ws.Holder;");
	ifaceWriter.println("import javax.jws.soap.*;");
	ifaceWriter.println("import javax.xml.ws.soap.Addressing;");
	ifaceWriter.println("import javax.annotation.Resource;");

	if (service.isAddressingEnabled()) {
	    ifaceWriter.println("@Addressing(enabled=true)");
	}
	ifaceWriter.println("@WebService(name = \"" + service.getName() + "\", " + "targetNamespace = \""
		+ service.getNamespace() + "\")");

	ifaceWriter.println("@SOAPBinding(style = SOAPBinding.Style." + service.getBindingStyle() + ","
		+ " use = SOAPBinding.Use." + service.getBindingUse() + ", "
		+ "parameterStyle = SOAPBinding.ParameterStyle." + service.getBindingParamStyle() + ")");

	ifaceWriter.println("interface I" + service.getName() + " {");

	if (service.isAddressingEnabled()) {
	    classWriter.println("@Addressing(enabled=true)");
	}
	classWriter.println("@WebService(name = \"" + service.getName() + "\", " + "portName = \""
		+ service.getPortName() + "\", " + "targetNamespace = \"" + service.getNamespace() + "\", "
		+ "endpointInterface = \"I" + service.getName() + "\")");

	classWriter.println("class " + service.getName() + " implements I" + service.getName() + " {");

	tabuList.add(service.getName());

	String wscVarName = generateVariableName("wsc", tabuList);

	classWriter.println("\t@Resource");
	classWriter.println("\t" + WebServiceContext.class.getName() + " " + wscVarName + ";");

	String wsVarName = generateVariableName("ws", tabuList);

	classWriter.println("\t" + WebService.class.getName() + " " + wsVarName + "="
		+ AbstractModelElement.class.getName() + ".getElementMap(" + WebService.class.getName() + ").get(\""
		+ wsm.getId() + "\");");

	classWriter.println("");
	classWriter.println("\tpublic " + service.getName() + "() throws Exception {");

	String deplHookVarName = generateVariableName("depl", tabuList);

	classWriter.println("\t\t" + Closure.class.getName() + " " + deplHookVarName + "=" + wsVarName
		+ ".getOnDeploy();");
	classWriter.println("\t\tif (" + deplHookVarName + ") {");
	classWriter.println("\t\t\t" + wsVarName + ".getPlugin().prepareHookClosure(" + wscVarName + "," + wsVarName
		+ "," + deplHookVarName + ").call();");
	classWriter.println("\t\t}");

	classWriter.println("\t}");

	classWriter.println("");
	classWriter.println("\tpublic void onUndeploy() {");

	String undeplHookVarName = generateVariableName("undepl", tabuList);

	classWriter.println("\t\t" + Closure.class.getName() + " " + undeplHookVarName + "=" + wsVarName
		+ ".getOnUndeploy();");
	classWriter.println("\t\tif (" + undeplHookVarName + ") {");
	classWriter.println("\t\t\t" + wsVarName + ".getPlugin().prepareHookClosure(" + wscVarName + "," + wsVarName
		+ "," + undeplHookVarName + ").call();");
	classWriter.println("\t\t}");

	classWriter.println("\t}");

	for (WsOperation operation : service.getOperations()) {

	    DataType returnType = operation.getReturnType();

	    DataType responseParam = operation.getParamTypes().get(operation.getName() + "Response");

	    if (responseParam == null) {
		responseParam = operation.getParamTypes().get(operation.getName() + "Result");
	    }

	    boolean responseTypeSwitched = false;

	    if (responseParam != null && returnType.getJavaClass() == void.class) {
		returnType = responseParam;
		responseTypeSwitched = true;
	    }

	    ifaceWriter.println("");
	    ifaceWriter.println("\t@WebMethod");
	    ifaceWriter.println("\t@WebResult(name=\"response\")");
	    ifaceWriter.print("\tpublic " + returnType.extractClassName() + " " + operation.getName() + "(");

	    classWriter.println("");
	    classWriter.println("\t@WebMethod");
	    classWriter.println("\t@WebResult(name=\"response\")");
	    classWriter.print("\tpublic " + returnType.extractClassName() + " " + operation.getName() + "(");

	    tabuList.add(operation.getName());

	    Map<String, DataType> paramTypes = operation.getParamTypes();

	    Iterator<String> parIter = paramTypes.keySet().iterator();
	    List<String> parNames = new ArrayList<String>();

	    HashSet<String> holderNames = new HashSet<String>();
	    HashSet<String> noInputNames = new HashSet<String>();

	    boolean isFirst = true;

	    while (parIter.hasNext()) {
		String name = parIter.next();

		if (responseTypeSwitched) {
		    if (name.equals(operation.getName() + "Response") || name.equals(operation.getName() + "Result")) {
			continue;
		    }
		}

		if (!isFirst) {
		    ifaceWriter.print(", ");
		    classWriter.print(", ");
		}

		isFirst = false;

		DataType type = paramTypes.get(name);

		ifaceWriter.print("@WebParam(name=\"" + name + "\"");
		classWriter.print("@WebParam(name=\"" + name + "\"");

		if (type.isHeader()) {
		    ifaceWriter.print(",header=true, mode=WebParam.Mode." + type.getHeaderMode());
		    classWriter.print(",header=true, mode=WebParam.Mode." + type.getHeaderMode());
		}

		if (type.isOutHeader() || type.isInOutHeader()) {
		    ifaceWriter.print(") Holder<" + type.extractClassName() + "> " + name);
		    classWriter.print(") Holder<" + type.extractClassName() + "> " + name);
		    holderNames.add(name);
		} else {
		    ifaceWriter.print(") " + type.extractClassName() + " " + name);
		    classWriter.print(") " + type.extractClassName() + " " + name);
		}

		if (type.isOutHeader()) {
		    noInputNames.add(name);
		}

		tabuList.add(name);
		parNames.add(name);
	    }

	    ifaceWriter.println(") throws Exception;");

	    classWriter.println(") throws Exception {");

	    String resultName = generateVariableName("result", tabuList);

	    classWriter.println("\t\tdef " + resultName + ";");

	    String opVarName = generateVariableName("op", tabuList);

	    classWriter.println("\t\t" + WsOperation.class.getName() + " " + opVarName + "=" + wsVarName
		    + ".getOperation(\"" + operation.getName() + "\");");

	    String beVarName = generateVariableName("be", tabuList);

	    classWriter.println("\t\t" + Closure.class.getName() + " " + beVarName + "=" + opVarName
		    + ".getBehavior();");

	    classWriter.println("\t\tif (" + beVarName + ") {");

	    String argArray = "[";
	    String argNameArray = "[";

	    classWriter.println("\t\t\t" + beVarName + "=" + wsVarName + ".getPlugin().prepareOperationClosure("
		    + wscVarName + "," + wsVarName + "," + opVarName + ");");

	    for (String parName : parNames) {

		if (noInputNames.contains(parName)) {
		    continue;
		}

		argNameArray += "\"" + parName + "\",";

		if (holderNames.contains(parName)) {
		    argArray += parName + ".value,";
		    classWriter.println("\t\t\t" + ClosureUtil.class.getName() + ".getInstance().setVariable("
			    + beVarName + ",\"" + parName + "\"," + parName + ".value);");
		} else {
		    argArray += parName + ",";
		    classWriter.println("\t\t\t" + ClosureUtil.class.getName() + ".getInstance().setVariable("
			    + beVarName + ",\"" + parName + "\"," + parName + ");");
		}
	    }

	    argArray += "]";
	    argNameArray += "]";

	    classWriter.println("\t\t\t" + ClosureUtil.class.getName() + ".getInstance().registerConstant(" + beVarName
		    + ",\"global\"," + wsVarName + ".getPlugin().getServiceVariableBinding(" + wsVarName + ")" + ");");

	    classWriter.println("\t\t\t" + ClosureUtil.class.getName() + ".getInstance().registerConstant(" + beVarName
		    + ",\"args\"," + argArray + ");");
	    classWriter.println("\t\t\t" + ClosureUtil.class.getName() + ".getInstance().registerConstant(" + beVarName
		    + ",\"argNames\"," + argNameArray + ");");

	    if (returnType.getJavaClass() != void.class) {
		classWriter.println("\t\t\t" + resultName + "=" + beVarName + ".call();");
	    } else {
		classWriter.println("\t\t\t" + beVarName + ".call();");
	    }

	    for (String holderName : holderNames) {
		classWriter.println("\t\t\t" + holderName + ".value=" + beVarName + "." + holderName + ";");
	    }

	    classWriter.println("\t\t}");

	    if (operation.isDummy()) {
		classWriter.println("\t\tif (" + resultName + " == null) {");
		classWriter.println("\t\t\t" + resultName + "=" + opVarName + ".getReturnType().createDummy();");
		classWriter.println("\t\t}");
	    }

	    if (returnType.getJavaClass() != void.class) {
		classWriter.println("\t\treturn " + resultName + ";");
	    }
	    classWriter.println("\t}");

	}

	ifaceWriter.println("}");
	classWriter.println("}");

	classWriter.println("return new " + service.getName() + "()");

	classWriter.flush();

	GroovyShell shell = new GroovyShell(DataType.LOADER);

	// System.out.println(ifaceStringWriter.toString() + "\n" +
	// classStringWriter.toString());

	return shell.evaluate(ifaceStringWriter.toString() + "\n" + classStringWriter.toString());
    }

    /**
     * Creates a closure as binding environment for deployment/undeployment
     * hooks.
     * 
     * @param webServiceContext Context.
     * @param service Service.
     * @param code Closure code.
     * @param bindings Further bindings.
     * @return Closure.
     */
    public Closure prepareHookClosure(WebServiceContext webServiceContext, WebService service, Closure code,
	    ExecutionEnvironment... bindings) {

	if (code == null) {
	    return null;
	}

	Closure result = (Closure) code.clone();

	CLOSURE_UTIL.setPrimaryDispatcher(result, new ExecutionEnvironment());
	CLOSURE_UTIL.registerConstant(result, "global", getServiceVariableBinding(service));
	CLOSURE_UTIL.attachDelegates(result, service, Host.getLocalhost());

	return result;
    }

    /**
     * Creates a closure as binding environment for deployment/undeployment
     * hooks.
     * 
     * @param webServiceContext Context.
     * @param service Service.
     * @param operation Operation.
     * @param bindings Further bindings.
     * @return Closure.
     */
    public Closure prepareOperationClosure(WebServiceContext webServiceContext, WebService service,
	    WsOperation operation, ExecutionEnvironment... bindings) {

	Closure behavior = operation.getBehavior();

	if (behavior == null) {
	    return null;
	}

	CLOSURE_UTIL.setPrimaryDispatcher(behavior, new ExecutionEnvironment());
	CLOSURE_UTIL.registerConstant(behavior, "global", getServiceVariableBinding(service));
	CLOSURE_UTIL.attachDelegates(behavior, operation, service, Host.getLocalhost());

	return behavior;
    }

    /**
     * Returns the variable binding of a Web service.
     * 
     * @param service Web service.
     * @return Binding.
     */
    public Binding getServiceVariableBinding(WebService service) {

	synchronized (serviceVarBindings) {
	    Binding b = serviceVarBindings.get(service);
	    if (b == null) {
		b = new ExecutionEnvironment();
		serviceVarBindings.put(service, b);
	    }
	    return b;
	}
    }

    private String generateVariableName(String prefix, Collection<String> tabuList) {

	String varName = null;
	String pref = prefix;

	if (pref == null) {
	    pref = "var";
	}

	int i = 0;
	do {
	    varName = pref + i;
	    i++;
	} while (tabuList.contains(varName));

	tabuList.add(varName);
	return varName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#undeployElement(
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    public void undeployElement(WebService e) {

	Object wsi = deployedServices.remove(e);

	if (wsi != null) {
	    try {
		InvokerHelper.invokeMethod(wsi, "onUndeploy", null);
	    } catch (Exception ex) {
		logger.warn("Service '" + e.getName() + "' threw exception during undeployment.", ex);
	    }
	}

	Server s = deployedEndpoints.remove(e);

	if (s != null) {
	    s.stop();
	}

	serviceVarBindings.remove(e);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getArtifact(java
     * .lang.Class, at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * java.lang.String[])
     */
    @Override
    public <S> S getArtifact(Class<S> type, WebService modelelement, String... additionalIDs) {

	if (type.equals(Server.class)) {
	    try {
		return (S) deployedEndpoints.get(modelelement);
	    } catch (NullPointerException npe) {
		return null;
	    }
	}

	if (type.equals(Endpoint.class)) {
	    try {
		return (S) getArtifact(Server.class, modelelement).getEndpoint();
	    } catch (NullPointerException npe) {
		return null;
	    }
	}

	try {
	    return (S) deployedServices.get(modelelement);
	} catch (NullPointerException npe) {
	}

	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStart()
     */
    @Override
    protected void doStart() {

    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#doStop()
     */
    @Override
    protected void doStop() {

	for (WebService ws : new ArrayList<WebService>(deployedServices.keySet())) {
	    undeployElement(ws);
	}
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getAliases()
     */
    @Override
    public List<String> getAliases() {
	List<String> aliases = new ArrayList<String>();
	aliases.add("wsgen");
	return aliases;
    }

    /*
     * (non-Javadoc)
     * 
     * @see at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getCommands()
     */
    @Override
    public Map<String, Closure> getCommands() {
	return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.plugin.AGenesis2Plugin#getTypesAndBuilders
     * ()
     */
    @Override
    public Map<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> getTypesAndBuilders() {

	Map<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>> result = new HashMap<Class<? extends AbstractModelElement>, Class<? extends AbstractModelBuilder<? extends AbstractModelElement>>>();

	result.put(WebService.class, WebServiceBuilder.class);
	result.put(WsOperation.class, WsOperationBuilder.class);

	return result;
    }
}
