package at.ac.tuwien.vitalab.genesis2.plugin.wsgen;

import groovy.lang.Closure;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import at.ac.tuwien.vitalab.genesis2.core.model.AbstractModelElement;
import at.ac.tuwien.vitalab.genesis2.core.model.ElementMap;
import at.ac.tuwien.vitalab.genesis2.core.model.builder.AbstractModelBuilder;
import at.ac.tuwien.vitalab.genesis2.core.model.types.DataType;

/**
 * Builder for WsOperation type.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public class WsOperationBuilder extends AbstractModelBuilder<WsOperation> {

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.builder.AModelBuilder#attachChild
     * (at.ac.tuwien.vitalab.genesis2.core.model.AModelElement,
     * at.ac.tuwien.vitalab.genesis2.core.model.AModelElement)
     */
    @Override
    protected void attachOrphanedChild(WsOperation parent, AbstractModelElement child) {

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.builder.AModelBuilder#processArgs
     * (java.util.List)
     */
    @Override
    protected List<?> processArgs(List<?> args) {

	List<Object> result = new ArrayList<Object>();

	Map<Object, Object> m = new HashMap<Object, Object>();

	result.add(m);

	for (Object a : args) {

	    if (a instanceof Map) {
		m.putAll(processMap((Map<Object, Object>) a));
	    } else if (a instanceof Closure) {
		m.put("behavior", a);
	    } else {
		result.add(a);
	    }
	}

	return result;
    }

    private Map<Object, Object> processMap(Map<?, ?> map) {

	ElementMap<DataType> datatypeMap = AbstractModelElement.getElementMap(DataType.class);

	Map<Object, Object> result = new HashMap<Object, Object>();
	Map<String, DataType> parameters = new LinkedHashMap<String, DataType>();

	for (Object k : map.keySet()) {

	    if ("response".equals(k)) {
		result.put("returnType", datatypeMap.create(map.get(k)));
	    } else {
		Object value = map.get(k);

		if (value != null && (value instanceof DataType || value instanceof Class)) {
		    parameters.put(k.toString(), datatypeMap.create(value));
		} else {
		    result.put(k.toString(), value);
		}
	    }
	}

	result.put("paramTypes", parameters);

	return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * at.ac.tuwien.vitalab.genesis2.core.model.builder.AModelBuilder#nextChildType
     * (java.lang.Object, java.lang.Object, java.util.Map)
     */
    @Override
    protected Class<? extends AbstractModelElement> nextChildType(Object name, Object value, Map<?, ?> attributes) {
	return DataType.class;
    }

}
