package at.ac.tuwien.vitalab.genesis2.utils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Invocation utilities.
 * 
 * @author Lukasz Juszczyk
 * 
 */
public final class G2InvokerUtil {

    /**
     * Static instance.
     */
    private static final G2InvokerUtil INSTANCE = new G2InvokerUtil();

    /**
     * Constructor.
     */
    private G2InvokerUtil() {
	// nothing
    }

    /**
     * Returns a singleton instance.
     * 
     * @return Instance.
     */
    public static G2InvokerUtil getInstance() {
	return INSTANCE;
    }

    /**
     * Invoke a real method on a Java/Groovy object, by passing the dispatchers.
     * 
     * @param o Target object.
     * @param name Method name.
     * @param args Method args.
     * @return Result.
     * 
     * @throws IllegalArgumentException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    public Object invokeRealMethod(Object o, String name, Object... args) throws IllegalArgumentException,
	    IllegalAccessException, InvocationTargetException, NoSuchMethodException {

	Method method = null;

	for (Method m : o.getClass().getDeclaredMethods()) {
	    if (!m.getName().equals(name)) {
		continue;
	    }

	    Class<?>[] params = m.getParameterTypes();

	    if (params.length != args.length) {

		if (method != null && params.length == 1 && params[0].equals(Object.class)) {
		    method = m;
		}

		continue;
	    }

	    for (int i = 0; i < params.length; i++) {
		if (params[i] != args[i]) {
		    continue;
		}
	    }

	    method = m;
	}

	if (method != null) {
	    return method.invoke(o, args);
	}

	throw new NoSuchMethodException("Cannot find method " + name + "(" + args + ")");
    }
}
